/*
    Copyright 2013 Hendrik Lipka
    
    This file is part of the MetaWatch client.

    MetaWatch client is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MetaWatch client is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MetaWatch client.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.hendriklipka.metawatch.linux;

import purejavacomm.*;

import purejavacomm.CommPortIdentifier;
import purejavacomm.SerialPort;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Calendar;
import java.util.Date;

public class Main
{

    public static void main(String[] args)
    {
        String port="/dev/rfcomm0";
        if (0!=args.length)
            port=args[0];
        try
        {
            CommPortIdentifier cpi = CommPortIdentifier.getPortIdentifier(port);
            SerialPort sp = (SerialPort) cpi.open("linux_mwc", 1000);
            OutputStream os = sp.getOutputStream();
            InputStream is = sp.getInputStream();

            getID(os, is);
            getVoltage(os, is);
            sendRtcNow(os, is);
            // set to 24h mode
//            setNVByteValue(os, is, 0x2009, (byte) 1);
            // set date display to day first
//            setNVByteValue(os, is, 0x200a,(byte)1);
            // measure battery once per minute
//            setNVIntValue(os, is, 0x2003,(int)60);
        }
        catch (NoSuchPortException e)
        {
            e.printStackTrace();
        }
        catch (PortInUseException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    private static void getID(final OutputStream os, final InputStream is) throws IOException
    {
        byte[] getid = {0x01, 0x06, 0x01, 0x00};

        byte[] msg = createMessage(getid);
        os.write(msg);

        int b = is.read();
        if (b != 0x01)
        {
            System.err.println("wrong answer: " + b);
            return;
        }
        int len = is.read(); // length
        int type = is.read();
        if (type!=0x02)
        {
            System.err.println("wrong response type");
        }
        is.read(); // options
        b=is.read();
        System.out.println("type="+b);
        is.read(); // crc
        is.read();
    }

    private static void getVoltage(final OutputStream os, final InputStream is) throws IOException
    {
        byte[] getids = {0x01, 0x06, 0x56, 0x00};
        byte[] msg = createMessage(getids);

        os.write(msg);

        int b = is.read();
        if (b != 0x01)
        {
            System.err.println("wrong answer: " + b);
            return;
        }
        int len = is.read(); // length
        int type = is.read();
        if (type!=0x57)
        {
            System.err.println("wrong response type");
        }
        is.read(); // options
        byte[] result = new byte[8];
        for (int i = 0; i < len - 4; i++)
        {
            int c = is.read();
            result[i] = (byte) c;
        }
        int volt = result[4] + result[5] * 256;
        System.out.println("battery voltage=" + (float) volt / 1000);
    }

    public static void sendRtcNow(final OutputStream os, final InputStream is) throws IOException
    {
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int year = calendar.get(Calendar.YEAR);

        byte[] setRTC = new byte[12];

        setRTC[0] = 1;
        setRTC[1] = (byte) (setRTC.length + 2); // length
        setRTC[2] = 0x26;
        setRTC[3] = 0x00; // not used

        setRTC[4] = (byte) (year / 256);
        setRTC[5] = (byte) (year % 256);
        setRTC[6] = (byte) (calendar.get(Calendar.MONTH) + 1);
        setRTC[7] = (byte) calendar.get(Calendar.DAY_OF_MONTH);
        setRTC[8] = (byte) (calendar.get(Calendar.DAY_OF_WEEK) - 1);
        setRTC[9] = (byte) calendar.get(Calendar.HOUR_OF_DAY);
        setRTC[10] = (byte) calendar.get(Calendar.MINUTE);
        setRTC[11] = (byte) calendar.get(Calendar.SECOND);

        byte[] msg = createMessage(setRTC);

        os.write(msg);
        System.out.println("set RTC OK");

        // no response to this!
    }

    static void setNVByteValue(final OutputStream os, final InputStream is, int id, byte value) throws IOException
    {
        System.out.println("set NV value for id "+Integer.toHexString(id)+"="+value);
        byte[] nv={0x01, 0x0a, 0x30, 0x02, 0x00, 0x00, 0x00, 0x00};
        nv[4]=(byte)(id%256);
        nv[5]=(byte)(id>>8);
        nv[6]=1;
        nv[7]=value;

        byte[] msg=createMessage(nv);

        os.write(msg);

        int b = is.read();
        if (b != 0x01)
        {
            System.err.println("wrong answer: " + b);
            return;
        }
        int len = is.read(); // length
        int type = is.read();
        if (type!=0x31)
        {
            System.err.println("wrong response type");
            return;
        }
        int opt=is.read(); // options
        if (opt!=0)
        {
            System.out.println("error - options="+opt);
        }
        int idl=is.read();
        int idh=is.read();
        is.read(); // crc
        is.read();
        System.out.println("OK for "+Integer.toHexString(idh*256+idl));
    }

    static void setNVIntValue(final OutputStream os, final InputStream is, int id, int value) throws IOException
    {
        System.out.println("set NV value for id "+Integer.toHexString(id)+"="+value);
        byte[] nv={0x01, 0x0b, 0x30, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00};
        nv[4]=(byte)(id%256);
        nv[5]=(byte)(id>>8);
        nv[6]=(byte)2;
        nv[7]=(byte)(value&0xff);
        nv[8]=(byte)(value>>8);

        byte[] msg=createMessage(nv);

        os.write(msg);

        int b = is.read();
        if (b != 0x01)
        {
            System.err.println("wrong answer: " + b);
            return;
        }
        int len = is.read(); // length
        int type = is.read();
        if (type!=0x31)
        {
            System.err.println("wrong response type");
            return;
        }
        int opt=is.read(); // options
        if (opt!=0)
        {
            System.out.println("error - options="+opt);
        }
        int idl=is.read();
        int idh=is.read();
        is.read(); // crc
        is.read();
        System.out.println("OK for "+Integer.toHexString(idh*256+idl));
    }

    private static byte[] createMessage(final byte[] bytes)
    {
        final int length = bytes.length;
        byte[] msg = new byte[length + 2];
        byte[] crc = crc(bytes);
        System.arraycopy(bytes, 0, msg, 0, length);
        msg[length] = crc[0];
        msg[length + 1] = crc[1];
        return msg;
    }

    // code taken from the android MWM
    // org.metawatch.manager.Protocol.java
    // method byte[] crc(byte[]);
    public static byte[] crc(byte[] bytes)
    {
        byte[] result = new byte[2];
        short crc = (short) 0xFFFF;
        for (int j = 0; j < bytes.length; j++)
        {
            byte c = bytes[j];
            for (int i = 7; i >= 0; i--)
            {
                boolean c15 = ((crc >> 15 & 1) == 1);
                boolean bit = ((c >> (7 - i) & 1) == 1);
                crc <<= 1;
                if (c15 ^ bit)
                {
                    crc ^= 0x1021; // 0001 0000 0010 0001 (0, 5, 12)
                }
            }
        }
        int crc2 = crc - 0xffff0000;
        result[0] = (byte) (crc2 % 256);
        result[1] = (byte) (crc2 / 256);
        return result;
    }
}
